import sys
from typing import Callable
import threading
import time
from nmotion_transport import USBInterface, getValidUSBInterfaces
from utils.logger import Logger
#TODO: Need to check with MacOS <- valid for the cpp layer as well

class DeviceManager:
    __interfaces: 'dict[str, USBInterface]' = {}
    __stop_loop: bool = False
    __dev_manager_thread_f = None
    __connected_devices_on_last_update: 'dict[int, int]' = {}
    __on_new_dev_cb = lambda *args: None
    __on_remove_dev_cb = lambda *args: None
    
    def __init__(self, logger: Logger, on_new_device_cb: Callable[[int], None] = None, on_remove_device_cb: Callable[[int], None] = None) -> None:
        self.__logger = logger
        if(on_new_device_cb):
            self.__on_new_dev_cb = on_new_device_cb
        if(on_remove_device_cb):
            self.__on_remove_dev_cb = on_remove_device_cb        
        self.__dev_manager_thread_f = threading.Thread(target=self.__devManagerThread)
        self.__dev_manager_thread_f.start()
    
    def __initialiseInterface(self, interface_name) -> USBInterface:
        interface = USBInterface(interface_name)       
        self.__logger.notify(f'Connected to NLink device at {interface_name}.')    
        return interface
    
    def __removeInterface(self, interface_name):
        self.__logger.warn(f'NLink device removed at {interface_name}')
        try:
            # remove all devices connected to the interface
            for dev in self.__interfaces[interface_name].getConnectedDevices():
                self.__on_remove_dev_cb(dev, self.__connected_devices_on_last_update[dev])           
                self.__connected_devices_on_last_update.pop(dev)
            
            # iface = self.__interfaces.pop(interface_name)
            # iface.close()       
            self.__interfaces[interface_name].close()
            del self.__interfaces[interface_name]
        except Exception as err:
            print(f'Exception on remove interface: {err}')
              
        # TODO: del causes terminate called after throwing an instance of Serial::SerialException <- make serial library
        # more fault tolerant
        # try: 
        #     del self.__interfaces[interface_name]
        # except Exception as err:
        #     print(f'Exception on del: {err}')       
       
    def __devManagerThread(self):
        while(not self.__stop_loop):
            self.__updateInterfaces()
            self.__updateDevices()
            time.sleep(0.5)
    
    def __updateInterfaces(self):
        current_nlink_ports = getValidUSBInterfaces()
        
        # get diff
        nlink_ports_on_last_update = list(self.__interfaces.keys())
        newly_added_ports = list(set(current_nlink_ports) - set(nlink_ports_on_last_update))
        removed_ports = list(set(nlink_ports_on_last_update) - set(current_nlink_ports))
        
        # add to dict
        for port in newly_added_ports:
            self.__interfaces[port] = self.__initialiseInterface(f'{port}')
                      
        # remove from dict
        for port in removed_ports:
            self.__removeInterface(port)
            
    def __updateDevices(self):
        current_connected_devices = {}
        for iface in self.__interfaces.values():
            current_connected_devices.update(iface.getConnectedDevices())
            # current_connected_devices = current_connected_devices + list(iface.getConnectedDevices().keys())
                        
        # Diff between this and __connected_devices_on_last_update list and call necessary functions
        newly_added_devices = list(set(current_connected_devices.keys()) - set(self.__connected_devices_on_last_update.keys()))
        removed_devices = list(set(self.__connected_devices_on_last_update.keys()) - set(current_connected_devices.keys()))
        
        for dev in newly_added_devices:
            for iface in self.__interfaces.values():
                connected_devs = iface.getConnectedDevices();
                if(dev in connected_devs.keys()):
                    self.__on_new_dev_cb(dev, connected_devs[dev], iface)
        
        for dev in removed_devices:
            self.__on_remove_dev_cb(dev, self.__connected_devices_on_last_update[dev])
        
        self.__connected_devices_on_last_update = current_connected_devices    
                        
    def stopThreads(self):
        self.__stop_loop = True        
        
    def __del__(self):
        self.__stop_loop = True
        if(self.__dev_manager_thread_f):
            if(self.__dev_manager_thread_f.is_alive()):
                self.__dev_manager_thread_f.join()